<?php 
/**
 * Controller class for admin pages
 */
class AdminController
{
	
	function __construct(){
		
		add_action( 'admin_menu', [ $this, 'blocked_in_china_page_callback' ] );

		add_action( 'admin_enqueue_scripts', [ $this, 'blocked_in_china_scripts_enqueue' ] );

		add_filter( 'cron_schedules', [ $this, 'blocked_in_china_cron_add_monthly' ] );
 		
		add_action("wp_ajax_gpbic_run_api", [ $this, 'blocked_in_china_run_api' ] );
		add_action("wp_ajax_nopriv_gpbic_run_api", [ $this, 'blocked_in_china_run_api' ] );

		add_action("wp_ajax_bic_filter_log", [ $this, 'blocked_in_china_filter_log' ] );
		add_action("wp_ajax_nopriv_bic_filter_log", [ $this, 'blocked_in_china_filter_log' ] );

		add_action("wp_ajax_gpbic_run_api_manual", [ $this, 'blocked_in_china_run_api_manual' ] );
		add_action("wp_ajax_nopriv_gpbic_run_api_manual", [ $this, 'blocked_in_china_run_api_manual' ] );

		add_action("wp_ajax_bic_admin_status_bar", [ $this, 'blocked_in_china_admin_status_bar' ] );
		add_action("wp_ajax_nopriv_bic_admin_status_bar", [ $this, 'blocked_in_china_admin_status_bar' ] );

		if( get_option('bic_admin_bar_status') ){
			add_action( 'admin_bar_menu', [ $this, 'admin_bar_item' ], 500 );
			add_action('admin_head', [ $this, 'bic_custom_style' ]);
		}

		

	
	}

	public function blocked_in_china_cron_add_monthly(){
		$schedules['monthly'] = array(
		'interval' => MONTH_IN_SECONDS,
		'display' => __( 'Once Monthly' )
		);
		return $schedules;
	}

	public function blocked_in_china_filter_log(){
		
		if( !isset($_POST['option']) || empty($_POST['option']) ){
			wp_send_json_error( __("Please select an option to filter", 'blocked-in-china' ) );
		} else{
			$option = sanitize_text_field($_POST['option']);
		}

		global $wpdb;

		$query = $wpdb->prepare(
			"SELECT `option_value` FROM $wpdb->options WHERE `option_id` LIKE %d",
			$option
		);
		$record = $wpdb->get_var($query);
		$record = maybe_unserialize( $record );
		// print_r($record);
		if( $record ){
			if( $record['data']['summary']['result'] == "visible" ){
				wp_send_json_success( $record['data'] );
			} else{
				wp_send_json_error( $record['data']['summary']['description'] );
			}
		} else{
			wp_send_json_error( __("Unable to fetch the log. Reload the page and try again...", 'blocked-in-china' )  );
		}
	}

	public function blocked_in_china_run_api(){
		
		// verify nonce
		check_ajax_referer( 'gpbic_nonce', 'security' );

		global $bic_fs;
		
		// check if on a premium Freemius plan
		if( $bic_fs->is_plan( 'free', true ) ){
			wp_send_json_error(
				sprintf(
					__("Scheduling status updates is not allowed on %1$s plan, %2$s", 'blocked-in-china' ),
					'<b>' . ucfirst( __( 'Free', 'blocked-in-china' ) ) . '</b>',
					'<a href="' . esc_url( $bic_fs->get_upgrade_url() ) . '">' . __( 'Upgrade here', 'blocked-in-china' ) . '</a>'
				)
			);
		}

		if( !isset($_POST['bic_cron_on']) || empty($_POST['bic_cron_on']) ){
			
			// unschedule current CRON 
			if ( wp_next_scheduled( 'bic_cron_hook' ) ) {

				$timestamp = wp_next_scheduled( 'bic_cron_hook' );
				wp_unschedule_event( $timestamp, 'bic_cron_hook' );

			}

			delete_option( 'bic_cron_schedule' );
			$cron_status = false;

		} else{
			
			$cron_status = true;

			if( !isset($_POST['frequency']) || empty($_POST['frequency']) ){
				wp_send_json_error( __("Please select a frequency", 'blocked-in-china' ) );
			} else{
				$frequency = sanitize_text_field($_POST['frequency']);
			}

			// perform Freemius plan validations
			$show = false;
			if ( $bic_fs->is_plan( 'personal', true ) ) {
				$show = [ 'monthly', 'weekly' ];
			} else if ( $bic_fs->is_plan( 'pro', true ) ) {
				$show = [ 'monthly', 'weekly', 'daily' ];
			} else if ( $bic_fs->is_plan( 'agency', true ) ) {
				$show = [ 'monthly', 'weekly', 'daily', 'hourly' ];
			}

			if( !$show || !in_array( $frequency, $show ) ){
				wp_send_json_error(
					sprintf(
						__("You are not allowed to schedule %s updates, %1$s", 'blocked-in-china' ),
						$frequency,
						'<a href="' . esc_url( $bic_fs->get_upgrade_url() ) . '">' . __( 'Upgrade here', 'blocked-in-china' ) . '</a>'
					)
				);
			}

			// unschedule current CRON 
			if ( wp_next_scheduled( 'bic_cron_hook' ) ) {

				$timestamp = wp_next_scheduled( 'bic_cron_hook' );
				wp_unschedule_event( $timestamp, 'bic_cron_hook' );

			}

			// check if CRON is already set at same Frequency
			if( get_option( 'bic_cron_schedule' ) == $frequency ){
				wp_send_json_error(
					sprintf(
						__("Cron is already scheduled to run %1$s.", 'blocked-in-china' ),
						$frequency
					)
				);
			}

			// schedule new CRON
			$response = wp_schedule_event( time(), $frequency, 'bic_cron_hook' );
		    if($response){
		    	update_option( 'bic_cron_schedule', $frequency );
		    }

		}

		update_option( 'bic_cron_on', $cron_status );
		wp_send_json_success( __("Settings saved successfully!", 'blocked-in-china' ) );

	}

	/*Run API Manually*/
	public function blocked_in_china_run_api_manual(){
		
		// verify nonce
		check_ajax_referer( 'gpbic_nonce', 'security' );

		// validate the request
		$next_run_time = get_option( 'bic_manual_api_run' );
		$current_time = time();

		if( $current_time < $next_run_time ){
			wp_send_json_error( __( 'You are not authorized to perform the request now.', 'blocked-in-china' ) );
		}

		global $bic_fs;

		$next_time = strtotime("+30 day", time());

		if ( $bic_fs->is_plan( 'personal', true ) ) {
			$next_time = strtotime("+7 day", time());
		} else if ( $bic_fs->is_plan( 'pro', true ) ) {
			$next_time = strtotime("+1 day", time());
		} else if ( $bic_fs->is_plan( 'agency', true ) ) {
			$next_time = strtotime("+1 hour", time());
		}
		
		$return = bic_cron_execute_callback();

		if( $return['success'] ){
			get_option( 'bic_cron_log' .'-'. gmdate('Y-m-d H:i', strtotime($return['timestamp'])) );
			update_option( 'bic_manual_api_run', $next_time );
			update_option( 'bic_manual_api_run_last', strtotime($return['timestamp']) );
		    wp_send_json_success( $return['data'] );
		} else{
            wp_send_json_error( $return['data'] );
		}
	}


	/**
	 * Register blocked in China menu.
	 */
	public function blocked_in_china_page_callback() {
	    add_menu_page(
	        __( 'Blocked in China', 'gpbic' ),
	        'Blocked in China',
	        'manage_options',
	        'blocked-in-china',
	        [ $this, 'blocked_in_china_menu_callback' ],
	        'dashicons-unlock',
	        100
	    );
	}

	public function blocked_in_china_menu_callback(){
		if( isset($_GET['tab']) && $_GET['tab'] == "bundle" ){
			require_once GPBIC_PLUGIN_PATH . '/admin/templates/bundle.php';
		} else if( isset($_GET['tab']) && $_GET['tab'] == "log" ){
			require_once GPBIC_PLUGIN_PATH . '/admin/templates/log.php';
		} else if( isset($_GET['tab']) && $_GET['tab'] == "schedule" ){
			require_once GPBIC_PLUGIN_PATH . '/admin/templates/schedule.php';
		} else {
			require_once GPBIC_PLUGIN_PATH . '/admin/templates/form.php';
		}
	}

	public function blocked_in_china_scripts_enqueue( $hook ) {
		
		if ( $hook != "toplevel_page_blocked-in-china" ) {
	        return;
	    }

	    wp_enqueue_style( 'gpbic-style', GPBIC_PLUGIN_URL . '/assets/css/gpbic.css' );
	   	wp_enqueue_script(
	        'gpbic-script',
	        GPBIC_PLUGIN_URL . '/assets/js/gpbic.js',
	        [ 'jquery', 'wp-api-request' ],
	        GPBIC_PLUGIN_VERSION,
	        true
	    );
	    wp_localize_script(
	        'gpbic-script',
	        'gpbic',
	        array( 
	        	'ajaxurl' => admin_url( 'admin-ajax.php' ),
	        	'current_page' => isset($_GET["tab"]) ? $_GET["tab"] : '',
	        	'siteurl'	=> site_url(),
	        	'nonce' => wp_create_nonce('wp_rest'),
	        	)
	    );
 
	}

	public function blocked_in_china_admin_status_bar()
	{
		if( !isset($_POST['option']) || empty($_POST['option']) ){
			wp_send_json_error( __("Please select an option to filter", 'blocked-in-china' ) );
		}

		update_option( 'bic_admin_bar_status', $_POST['option']=='true' ? 1 : 0);

		return true;
	}
	public function admin_bar_item ( WP_Admin_Bar $admin_bar ) {
	    

	    $admin_item = array(
	        'id'    => 'bic-status',
	        'parent' => 'top-secondary',
	        'group'  => null,
			'href' => admin_url('admin.php?page=blocked-in-china')
	    );

	    $bic_manual_api_run = get_option( 'bic_manual_api_run' );
		$bic_manual_api_run_last = get_option( 'bic_manual_api_run_last' );
		$manual_log = get_option( 'bic_cron_log' .'-'. gmdate('Y-m-d H:i', $bic_manual_api_run_last) );
	    
		if($bic_manual_api_run && !empty($manual_log['data']['servers'])){

			$title = sprintf(
				__( 'Blocked in China Status: Not Blocked as of %1$s', 'blocked-in-china' ),
				gmdate('F d, Y', $bic_manual_api_run_last)
			);
			$meta = array(
				'title' => __( 'Blocked in China Status: Not Blocked', 'blocked-in-china' ),
				'class' => 'bic-green'
			);
		
	    } elseif($bic_manual_api_run && empty($manual_log['data']['servers'])){

			$title = sprintf(
				__( 'Blocked in China Status: Blocked as of %1$s', 'blocked-in-china' ),
				gmdate('F d, Y', $bic_manual_api_run_last)
			);
			$meta = array(
				'title' => __( 'Blocked in China Status: Blocked', 'blocked-in-china' ),
				'class' => 'bic-red'
			);

		} else{

			$title = __( 'Blocked in China Status: Check', 'blocked-in-china' );
			$meta = array(
				'title' => __( 'Blocked in China Status: Check', 'blocked-in-china' ),
				'class' => 'bic-gray'
			);
			
		}
	    
		$admin_item['title'] = $title;
		$admin_item['meta'] = $meta;

	    $admin_bar->add_menu( $admin_item );
	}

	public function bic_custom_style() {
	  echo '<style>
	    .bic-gray{ background: #434343 !important; }
		.bic-green{ background: #6aa84f !important; }
		.bic-red{ background: #cc0000 !important; }
	  </style>';
	}
    
	public function china_blocked_content_allowed_html_tags() {
		return [
			'a'      => [ 'href' => [], 'target' => [], 'alt' => [] ],
			'br'     => [],
			'video'  => [ 'width' => [], 'height' => [] ],
			'source' => [ 'src' => [], 'type' => [] ],
			'strong' => [ 'style' => [] ],
			'sub'    => [ 'style' => [] ],
			'sup'    => [ 'style' => [] ],
			's'      => [ 'style' => [] ],
			'i'      => [ 'style' => [] ],
			'u'      => [ 'style' => [] ],
			'span'   => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'h1'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'h2'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'h3'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'ol'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'ul'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'li'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'em'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'hr'     => [],
			'p'      => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
			'img'    => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'src' => [], 'alt' => [], 'href' => [], 'rel' => [], 'target' => [], 'value' => [], 'name' => [], 'width' => [], 'height' => [], 'data' => [], 'title' => [] ]
		];
	}
	
}

new AdminController;