<?php

/*
 * Plugin Name:     Blocked in China
 * Plugin URI:      https://blockedinchina.io
 * Description:     Run status checks to see if your website is available in mainland China.
 * Version:         1.0.6
 * Author:          China Plugins
 * Author URI:      https://chinaplugins.com/
 * License:         GPLv3
 * Text Domain:     blocked-in-china
 * Domain Path:     /languages
 */
defined( 'ABSPATH' ) || exit;
define( "GPBIC_PLUGIN_VERSION", '1.0.6' );
define( 'GPBIC_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
define( 'GPBIC_PLUGIN_URL', plugin_dir_url( __FILE__ ) );

if ( function_exists( 'bic_fs' ) ) {
    bic_fs()->set_basename( false, __FILE__ );
} else {
    
    if ( !function_exists( 'bic_fs' ) ) {
        // Create a helper function for easy SDK access.
        function bic_fs()
        {
            global  $bic_fs ;
            
            if ( !isset( $bic_fs ) ) {
                // Include Freemius SDK.
                require_once dirname( __FILE__ ) . '/freemius/start.php';
                $bic_fs = fs_dynamic_init( array(
                    'id'                             => '6351',
                    'slug'                           => 'blocked-in-china',
                    'premium_slug'                   => 'blocked-in-china-paid',
                    'type'                           => 'plugin',
                    'public_key'                     => 'pk_14f0843156b18459bb009f67e5138',
                    'is_premium'                     => false,
                    'has_addons'                     => false,
                    'has_paid_plans'                 => true,
                    'has_affiliation'                => 'selected',
                    'menu'                           => array(
                    'slug' => 'blocked-in-china',
                ),
                    'bundle_license_auto_activation' => true,
                    'is_live'                        => true,
                ) );
            }
            
            return $bic_fs;
        }
        
        // Init Freemius.
        bic_fs();
        // Signal that SDK was initiated.
        do_action( 'bic_fs_loaded' );
    }

}


if ( !function_exists( 'bic_after_license_change' ) ) {
    function bic_after_license_change( $change_name, FS_Plugin_Plan $plan )
    {
        // if ( in_array( $change_name, array( 'upgraded', 'changed' ) ) ) {
        // Plan was upgraded or changed.
        global  $wpdb ;
        $next_time = strtotime( "+30 day", time() );
        $bic_next_api_run_time = new DateTime( date( 'Y-m-d H:i:s', get_option( 'bic_manual_api_run_last' ) ) );
        $current_time = new DateTime( date( 'Y-m-d H:i:s', strtotime( "now" ) ) );
        $interval = $bic_next_api_run_time->diff( $current_time );
        $day_lapse = $interval->d;
        
        if ( bic_fs()->is_plan( 'personal', true ) ) {
            
            if ( $day_lapse >= 7 ) {
                $next_time = strtotime( "now" );
            } else {
                $time_occurs = 7 - $day_lapse;
                $next_time = strtotime( "+" . $time_occurs . " day", time() );
            }
            
            $current_plan = 'personal';
        } else {
            
            if ( bic_fs()->is_plan( 'pro', true ) ) {
                
                if ( $day_lapse >= 1 ) {
                    $next_time = strtotime( "now" );
                } else {
                    $next_time = strtotime( "+1 day", time() );
                }
                
                $current_plan = 'pro';
            } else {
                
                if ( bic_fs()->is_plan( 'agency', true ) ) {
                    $next_time = strtotime( "+1 hour", time() );
                    $current_plan = 'agency';
                }
            
            }
        
        }
        
        update_option( 'bic_manual_api_run', $next_time );
        update_option( 'bic_current_plan', $current_plan );
        update_option( 'bic_last_action', $change_name );
        // }
    }
    
    bic_fs()->add_action(
        'after_license_change',
        'bic_after_license_change',
        10,
        2
    );
}


if ( !function_exists( 'bic_load_translation' ) ) {
    /**
     * Register the plugin's translations files with WordPress.
     */
    function bic_load_translation()
    {
        load_plugin_textdomain( 'blocked-in-china', false, basename( dirname( __FILE__ ) ) . '/languages' );
    }
    
    add_action( 'plugins_loaded', 'bic_load_translation' );
}

// override Freemius optin message

if ( !function_exists( 'bic_fs_custom_connect_message_on_update' ) ) {
    function bic_fs_custom_connect_message_on_update(
        $message,
        $user_first_name,
        $product_title,
        $user_login,
        $site_link,
        $freemius_link
    )
    {
        return sprintf(
            __( 'Hey %1$s', 'my-text-domain' ) . ',<br>' . __( 'In order to use %2$s, it\'s necessary to share your website URL and other minimal data with our servers. Click below to agree to our  %3$s and connect to  %4$s and  %5$s' ),
            $user_first_name,
            '<b>' . $product_title . '</b>',
            '<a href="https://blockedinchina.io/terms-conditions-privacy-policy" target="_blank">Terms of Use & Privacy Policy</a>',
            '<a href="blockedinchina.io" target="_blank">blockedinchina.io</a>',
            '<a href="freemius.com" target="_blank">freemius.com</a>.'
        );
    }
    
    bic_fs()->add_filter(
        'connect_message_on_update',
        'bic_fs_custom_connect_message_on_update',
        10,
        6
    );
    bic_fs()->add_filter(
        'connect_message',
        'bic_fs_custom_connect_message_on_update',
        10,
        6
    );
}

// override Freemius optin message

if ( !function_exists( 'bic_fs_custom_connect_message_on_premium' ) ) {
    function bic_fs_custom_connect_message_on_premium( $message, $user_first_name, $product_title )
    {
        return sprintf(
            __( 'Hey %1$s', 'my-text-domain' ) . ',<br>' . __( 'In order to use %2$s, it\'s necessary to share your website URL and other minimal data with our servers. Enter your license key and click below to agree to our  %3$s and connect to  %4$s and  %5$s' ),
            $user_first_name,
            '<b>' . $product_title . '</b>',
            '<a href="https://blockedinchina.io/terms-conditions-privacy-policy" target="_blank">Terms of Use & Privacy Policy</a>',
            '<a href="blockedinchina.io" target="_blank">blockedinchina.io</a>',
            '<a href="freemius.com" target="_blank">freemius.com</a>.'
        );
    }
    
    bic_fs()->add_filter(
        'connect-message_on-premium',
        'bic_fs_custom_connect_message_on_premium',
        10,
        3
    );
}

// override Freemius opt-in logo

if ( !function_exists( 'bic_fs_custom_icon' ) ) {
    function bic_fs_custom_icon()
    {
        return dirname( __FILE__ ) . '/assets/images/icon-256x256.jpg';
    }
    
    bic_fs()->add_filter( 'plugin_icon', 'bic_fs_custom_icon' );
}

bic_fs()->override_i18n( array(
    'opt-in-connect' => __( "Yes - I'm in!", 'blocked-in-china' ),
) );
if ( function_exists( 'fs_override_i18n' ) ) {
    fs_override_i18n( array(
        'opt-in-connect' => __( 'Ok - I am in!', 'blocked-in-china' ),
    ), 'blocked-in-china' );
}
if ( !function_exists( 'bic_get_preemius_levels' ) ) {
    function bic_get_preemius_levels( $key, $swap = false )
    {
        $levels = array(
            'personal' => 'weekly',
            'pro'      => 'daily',
            'agency'   => 'hourly',
            'free'     => 'monthly',
        );
        if ( $swap ) {
            $levels = array_flip( $levels );
        }
        return $levels[$key];
    }

}
if ( !function_exists( 'bic_get_api_key' ) ) {
    function bic_get_api_key()
    {
        $url = esc_url( 'https://api.blockedinchina.io/wp-json/blocked-in-china-api/v1/latest-key?token=' . md5( gmdate( 'm' ) ) );
        // $url = esc_url('http://bic.local/wp-json/blocked-in-china-api/v1/latest-key?token='. md5( gmdate('m') ) );
        // $secured_url = wp_nonce_url( $url, 'bic_get_api' );
        $response = wp_remote_get( $url );
        
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            $return = array(
                'success' => false,
                'data'    => "Something went wrong: {$error_message}",
            );
        } else {
            $body = json_decode( wp_remote_retrieve_body( $response ), true );
            
            if ( isset( $body['data']['error'] ) ) {
                $return = array(
                    'success' => false,
                    'data'    => $body['data']['error'],
                );
            } else {
                $return = array(
                    'success' => true,
                    'data'    => $body['data']['api_key'],
                );
            }
        
        }
        
        return $return;
    }

}
add_action( 'bic_cron_hook', 'bic_cron_execute' );
if ( !function_exists( 'bic_cron_execute' ) ) {
    function bic_cron_execute()
    {
        bic_cron_execute_callback();
    }

}
if ( !function_exists( 'bic_cron_execute_callback' ) ) {
    function bic_cron_execute_callback()
    {
        $timestamp = gmdate( 'Y-m-d H:i:s' );
        global  $wpdb ;
        $api_key = bic_get_api_key();
        
        if ( !$api_key['success'] ) {
            $return = array(
                'timestamp' => $timestamp,
                'success'   => false,
                'data'      => $api_key['data'],
            );
        } else {
            $apiurl = add_query_arg( array(
                'domain' => site_url(),
                'apikey' => $api_key['data'],
                'output' => 'json',
            ), 'https://api.viewdns.info/chinesefirewall' );
            $response = wp_remote_get( $apiurl );
            $return = [];
            
            if ( is_wp_error( $response ) ) {
                $error_message = $response->get_error_message();
                $return = array(
                    'timestamp' => $timestamp,
                    'success'   => false,
                    'data'      => "Something went wrong: {$error_message}",
                );
            } else {
                $body = json_decode( wp_remote_retrieve_body( $response ), true );
                $return = array(
                    'timestamp' => $timestamp,
                    'success'   => true,
                    'data'      => [
                    'servers' => $body['response']['detail']['server'],
                    'summary' => $body['response']['summary'],
                ],
                );
            }
            
            $option_prefix = gmdate( 'Y-m-d H:i', strtotime( $timestamp ) );
            update_option( 'bic_cron_log' . '-' . $option_prefix, $return );
        }
        
        return $return;
    }

}
/**
 * Deactivation hook.
 */
if ( !function_exists( 'bic_deactivate' ) ) {
    function bic_deactivate()
    {
        global  $wpdb ;
        delete_option( 'bic_cron_schedule' );
        
        if ( wp_next_scheduled( 'bic_cron_hook' ) ) {
            $timestamp = wp_next_scheduled( 'bic_cron_hook' );
            wp_unschedule_event( $timestamp, 'bic_cron_hook' );
        }
    
    }

}
register_deactivation_hook( __FILE__, 'bic_deactivate' );
require_once GPBIC_PLUGIN_PATH . '/admin/class-admin-controller.php';